/* IMPORTANT: s necessari estar connectat amb l'usuari MOLIVERBES amb password SECRET */
/*            abans d'executar aquest script                                           */


/*****************************************************************************/
/*****************************************************************************/
/*            CREACIO DE FUNCIONS I PROCEDIMENTS EMMAGATZEMATS               */
/*****************************************************************************/
/*****************************************************************************/
/******************************************************************************
Nom   :   COMPROVACIO_DATA
Paquet:   TFG_PK_ABM
Funci:   Comprova si el mes i any de la data del sistema estan donats d'alta,
    	    en cas contrari els dona d'alta
********************************************************************************/
CREATE OR REPLACE FUNCTION COMPROVACIO_DATA
RETURN FLOAT
Is
param_input Varchar2(300);  
codi_error  Integer;
v_idMes     Integer;
v_idAny     Integer;
v_mes       Integer;
v_any       Integer;
num_reg     Integer;

Begin

  v_idMes := -1;
  v_idAny := -1;

-- Cerquem l'identificador del mes i any de la data del sistema
  SELECT EXTRACT(MONTH FROM SYSDATE), EXTRACT(YEAR FROM SYSDATE) INTO v_mes, v_any FROM DUAL;
  SELECT count(idAny) INTO num_reg FROM T_ANY WHERE anny = v_any;
-- Si no existeixen els donem d'alta
  IF num_reg = 0 THEN
	   INSERT INTO MOLIVERBES.T_ANY (Anny) VALUES (v_any) RETURNING idAny INTO v_idAny;
  ELSE
     SELECT idAny INTO v_idAny FROM T_ANY WHERE anny = v_any;
  END IF;
  
  SELECT count(idMes) INTO num_reg FROM T_MES WHERE Mes = v_mes AND idAny = v_idAny;
  IF num_reg = 0 THEN
	   INSERT INTO MOLIVERBES.T_MES (mes, idAny) VALUES (v_mes,v_idAny) RETURNING idMes INTO v_idMes;
  ELSE
     SELECT idMes INTO v_idMes FROM T_MES WHERE Mes = v_mes AND idAny = v_idAny;
  END IF;
-- Retornem l'identificador del mes  
  RETURN(v_idMes);
END;

/

/*******************************************************************************
Nom    :   PROCINS_BD_POBLACIO
Paquet :   TFG_PK_ABM
Funci :   Alta d'una nova poblaci
********************************************************************************/
Create Or Replace Procedure PROCINS_BD_POBLACIO (
  pPoblacio In   POBLACIO.Poblacio%Type,
  Rsp       Out  Nocopy Varchar2)
Is

param_input Varchar2(300);  
codi_error  Integer;

Begin
  param_input := 'pPoblacio='|| pPoblacio;
  Rsp := 'OK';
  
  INSERT INTO MOLIVERBES.POBLACIO(Poblacio) VALUES (pPoblacio);
  codi_error := SQLCODE;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;	
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCUPD_BD_POBLACIO
Paquet :   TFG_PK_ABM
Funci :   Modifica les dades d'una poblaci
********************************************************************************/
Create Or Replace Procedure  PROCUPD_BD_POBLACIO (
  pIdPoblacio In   POBLACIO.IdPoblacio%Type,
  pPoblacio   In   POBLACIO.Poblacio%Type,
  Rsp         Out  Nocopy Varchar2)
Is
param_input    Varchar2(300); 
codi_error     Integer;
reg_inexistent exception;

Begin
  param_input := 'pIdPoblacio=' || pIdPoblacio || ' pPoblacio=' || pPoblacio;
  Rsp := 'OK';

  UPDATE MOLIVERBES.POBLACIO SET Poblacio = pPoblacio WHERE IdPoblacio = pIdPoblacio;
  codi_error := SQLCODE;
  IF (SQL%ROWCOUNT = 0) THEN
     raise reg_inexistent;
  END IF;
  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;


Exception
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Poblaci inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;	
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCDEL_BD_POBLACIO
Paquet :   TFG_PK_ABM
Funci :   Elimina fsicament una poblaci
********************************************************************************/
Create Or Replace Procedure  PROCDEL_BD_POBLACIO (
  pIdPoblacio In   POBLACIO.IdPoblacio%Type,
  Rsp         Out  Nocopy Varchar2)
Is
param_input    Varchar2(300); 
codi_error     Integer;
reg_inexistent exception;

Begin
  param_input := 'pIdPoblacio=' || pIdPoblacio;
  Rsp := 'OK';

  DELETE FROM MOLIVERBES.POBLACIO WHERE IdPoblacio = pIdPoblacio;
  codi_error := SQLCODE;
  IF (SQL%ROWCOUNT = 0) THEN
     raise reg_inexistent;
  END IF;
  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Poblaci inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCINS_BD_TIPUSSERVEI
Paquet :   TFG_PK_ABM
Funci :   Alta d'un nou tipus de Servei
********************************************************************************/
Create Or Replace Procedure PROCINS_BD_TIPUSSERVEI (
  pTipus    In   TIPUSSERVEI.Tipus%Type,
  Rsp       Out  Nocopy Varchar2)
Is

param_input Varchar2(300);  
codi_error  Integer;

Begin
  param_input := 'pTipus='|| pTipus;
  Rsp := 'OK';

  INSERT INTO MOLIVERBES.TIPUSSERVEI(Tipus,data_creacio) VALUES (pTipus,comprovacio_data());
  codi_error := SQLCODE;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;	
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/


/*******************************************************************************
Nom    :   PROCUPD_BD_TIPUSSERVEI
Paquet :   TFG_PK_ABM
Funci :   Modifica les dades d'un tipus de Servei
********************************************************************************/
Create Or Replace Procedure  PROCUPD_BD_TIPUSSERVEI (
  pIdTipusServei In   TIPUSSERVEI.idTipusServei%Type,
  pTipus        In   TIPUSSERVEI.Tipus%Type,
  PVigencia     In   TIPUSSERVEI.Vigencia%Type,
  Rsp           Out  Nocopy Varchar2)
Is
param_input    Varchar2(300); 
codi_error     Integer;
reg_inexistent exception;

Begin
  param_input := 'pIdTipusServei=' || pIdTipusServei || ' pTipus=' || pTipus  || ' pVigencia=' || pVigencia;
  Rsp := 'OK';

  UPDATE MOLIVERBES.TIPUSSERVEI SET Tipus = pTipus, Vigencia = pVigencia WHERE IdTipusServei = pIdTipusServei;
  codi_error := SQLCODE;
  IF (SQL%ROWCOUNT = 0) THEN
     raise reg_inexistent;
  END IF;
  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Tipus de Servei inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCDEL_BD_TIPUSSERVEI
Paquet :   TFG_PK_ABM
Funci :   Elimina fsicament un tipus de Servei
********************************************************************************/
Create Or Replace Procedure  PROCDEL_BD_TIPUSSERVEI (
  pIdTipusServei In   TIPUSSERVEI.idTipusServei%Type,
  Rsp           Out  Nocopy Varchar2)
Is
param_input    Varchar2(300); 
codi_error     Integer;
reg_inexistent exception;

Begin
  param_input := 'pIdTipusServei=' || pIdTipusServei;
  Rsp := 'OK';

  DELETE FROM MOLIVERBES.TIPUSSERVEI WHERE IdTipusServei = pIdTipusServei;
  codi_error := SQLCODE;
  IF (SQL%ROWCOUNT = 0) THEN
     raise reg_inexistent;
  END IF;
  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Tipus de Servei inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCINS_BD_CATEGORIA
Paquet :   TFG_BD_ABM
Funci :   Alta d'una nova categoria
********************************************************************************/
Create Or Replace Procedure PROCINS_BD_CATEGORIA (
  pCategoria In   CATEGORIA.Categoria%Type,
  pRang      In   CATEGORIA.Rang%Type,
  pPunts     In   CATEGORIA.Punts%Type,
  Rsp        Out  Nocopy Varchar2)
Is

param_input Varchar2(300);  
codi_error  Integer;

Begin
  param_input := 'pCategoria='|| pCategoria || ' pRang=' || pRang || ' pPunts=' || pPunts;
  Rsp := 'OK';
  
  INSERT INTO MOLIVERBES.CATEGORIA(Categoria, Rang, Punts) VALUES (pCategoria, pRang, pPunts);
  codi_error := SQLCODE;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/


/*******************************************************************************
Nom    :   PROCUPD_BD_CATEGORIA
Paquet :   TFG_BD_ABM
Funci :   Modifica les dades d'una categoria
*******************************************************************************/
Create Or Replace Procedure  PROCUPD_BD_CATEGORIA (
  pIdCategoria In  CATEGORIA.IdCategoria%Type,
  pCategoria   In  CATEGORIA.Categoria%Type,
  pRang        In  CATEGORIA.Rang%Type,
  pPunts       In  CATEGORIA.Punts%Type,
  Rsp          Out  Nocopy Varchar2)
Is
param_input    Varchar2(300); 
codi_error     Integer;
reg_inexistent exception;

Begin
  param_input := 'pIdCategoria=' || pIdCategoria || ' pCategoria='|| pCategoria || ' pRang=' || pRang || ' pPunts=' || pPunts;
  Rsp := 'OK';

  UPDATE MOLIVERBES.CATEGORIA SET Categoria = pCategoria, Rang = pRang, Punts = pPunts WHERE IdCategoria = pIdCategoria;
  codi_error := SQLCODE;
  IF (SQL%ROWCOUNT = 0) THEN
     raise reg_inexistent;
  END IF;
  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Categoria inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/ 

/*******************************************************************************
Nom    :   PROCDEL_BD_CATEGORIA
Paquet :   TFG_BD_ABM
Funci :   Elimina fsicament una categoria
*******************************************************************************/
Create Or Replace Procedure  PROCDEL_BD_CATEGORIA (
  pIdCategoria In   CATEGORIA.IdCategoria%Type,
  Rsp          Out  Nocopy Varchar2)
Is
param_input    Varchar2(300); 
codi_error     Integer;
reg_inexistent exception;

Begin
  param_input := 'pIdCategoria=' || pIdCategoria;
  Rsp := 'OK';

  DELETE FROM MOLIVERBES.CATEGORIA WHERE IdCategoria = pIdCategoria;
  codi_error := SQLCODE;
  IF (SQL%ROWCOUNT = 0) THEN
     raise reg_inexistent;
  END IF;
  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Categoria inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/
/*******************************************************************************
Nom    :   PROCINS_BD_PERFIL
Paquet :   TFG_BD_ABM
Funci :   Alta d'un nou perfil
********************************************************************************/
Create Or Replace Procedure PROCINS_BD_PERFIL (
  pIdPerfil In   PERFIL.idperfil%Type,
  pDescrip  In   PERFIL.descrip%Type,
  Rsp       Out  Nocopy Varchar2)
Is

param_input Varchar2(300);  
codi_error  Integer;

Begin
  param_input := 'pIdPerfil= ' || pIdPerfil || 'pDescrip= ' || pDescrip;
  Rsp := 'OK';
  
  INSERT INTO MOLIVERBES.PERFIL(idperfil,descrip) VALUES (pIdPerfil,pDescrip);
  codi_error := SQLCODE;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);

  Commit;

Exception
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCUPD_BD_PERFIL
Paquet :   TFG_BD_ABM
Funci :   Modifica les dades d'un perfil
********************************************************************************/
Create Or Replace Procedure PROCUPD_BD_PERFIL (
  pIdPerfil In   PERFIL.idperfil%Type,
  pDescrip  In   PERFIL.descrip%Type,
  Rsp       Out  Nocopy Varchar2)
Is

param_input    Varchar2(300);  
codi_error     Integer;
reg_inexistent exception;

Begin
  param_input := 'pIdPerfil= ' || pIdPerfil || 'pDescrip= ' || pDescrip;
  Rsp := 'OK';
  
  UPDATE MOLIVERBES.PERFIL SET descrip = pDescrip WHERE idperfil = pIdPerfil;
  codi_error := SQLCODE;
  IF (SQL%ROWCOUNT = 0) THEN
     raise reg_inexistent;
  END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);

  Commit;

Exception
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Perfil inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCDEL_BD_PERFIL
Paquet :   TFG_BD_ABM
Funci :   Elimina fsicament un perfil
********************************************************************************/
Create Or Replace Procedure PROCDEL_BD_PERFIL (
  pIdPerfil In   PERFIL.idperfil%Type,
  Rsp       Out  Nocopy Varchar2)
Is

param_input    Varchar2(300);  
codi_error     Integer;
reg_inexistent exception;

Begin
  param_input := 'pIdPerfil= ' || pIdPerfil;
  Rsp := 'OK';
  
  DELETE FROM MOLIVERBES.PERFIL WHERE idperfil = pIdPerfil;
  codi_error := SQLCODE;
  IF (SQL%ROWCOUNT = 0) THEN
     raise reg_inexistent;
  END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);

  Commit;

Exception
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Perfil inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/
/*******************************************************************************
Nom    :   PROCINS_BD_EMPRESA
Paquet :   TFG_BD_ABM
Funci :   Alta d'una nova empresa
********************************************************************************/
Create Or Replace Procedure PROCINS_BD_EMPRESA (
  pNom           In   EMPRESA.Nom%Type,
  pAdreca        In   EMPRESA.Adreca%Type,
  pTelefon       In   EMPRESA.Telefon%Type,
  pIdPoblacio    In   EMPRESA.IdPoblacio%Type,
  pIdCategoria   In   EMPRESA.IdCategoria%Type,
  pNifPropietari In   EMPRESA.NifPropietari%Type,
  Rsp            Out  Nocopy Varchar2)
Is

param_input Varchar2(300);  
codi_error  Integer;

Begin
  param_input := 'pNom=' || pNom || ' pAdreca=' || pAdreca || 'pTelefon=' || pTelefon || ' pIdPoblacio=' || pIdPoblacio ||
		 'pIdCategoria=' || pIdCategoria || ' pNifPropietari=' || pNifPropietari;
  Rsp := 'OK';
  
  INSERT INTO MOLIVERBES.EMPRESA(nom, adreca, telefon, idPoblacio, idCategoria, nifPropietari,data_alta) 
                         VALUES (pNom,pAdreca,pTelefon,pIdPoblacio,pIdCategoria,pNifPropietari,comprovacio_data());
  codi_error := SQLCODE;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);

  Commit;

Exception
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/


/*******************************************************************************
Nom    :   PROCUPD_BD_EMPRESA
Paquet :   TFG_BD_ABM
Funci :   Modifica les dades d'una empresa
********************************************************************************/
Create Or Replace Procedure  PROCUPD_BD_EMPRESA (
  pIdEmpresa     In   EMPRESA.idEmpresa%Type,
  pNom           In   EMPRESA.Nom%Type,
  pAdreca        In   EMPRESA.Adreca%Type,
  pTelefon       In   EMPRESA.Telefon%Type,
  pIdPoblacio    In   EMPRESA.IdPoblacio%Type,
  pIdCategoria   In   EMPRESA.IdCategoria%Type,
  pNifPropietari In   EMPRESA.NifPropietari%Type,
  Rsp            Out  Nocopy Varchar2)
Is
param_input    Varchar2(300); 
codi_error     Integer;
num_reg        Integer;
reg_ja_baixa   exception;
reg_inexistent exception;

Begin
  param_input := 'pNom' || pNom || ' pAdreca=' || pAdreca || 'pTelefon=' || pTelefon || ' pIdPoblacio=' || pIdPoblacio ||
		 'pIdCategoria=' || pIdCategoria || ' pNifPropietari=' || pNifPropietari;

  Rsp := 'OK';

-- Mirem si existeix l'empresa
  SELECT count(idEmpresa) INTO num_reg FROM EMPRESA WHERE idEmpresa = pIdEmpresa;
  IF num_reg <> 0 THEN
-- Si existeix, feim l'actualitzaci si no est de baixa.  
	   UPDATE MOLIVERBES.EMPRESA SET Nom = pNom, Adreca = pAdreca, Telefon = pTelefon, idPoblacio = pIdPoblacio,
                                idCategoria = pIdCategoria, nifPropietari = pNifPropietari
                                WHERE idEmpresa = pIdEmpresa  AND data_baixa IS NULL;
	   codi_error := SQLCODE;
     IF (SQL%ROWCOUNT = 0) THEN
        raise reg_ja_baixa;
     END IF;
  ELSE
	   raise reg_inexistent;
  END IF;

  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);

  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_ja_baixa THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Empresa de baixa. No es pot modificar';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Empresa inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/


/*******************************************************************************
Nom    :   PROCDEL_BD_EMPRESA
Paquet :   TFG_BD_ABM
Funci :   Elimina una empresa (Baixa Lgica)
********************************************************************************/
Create Or Replace Procedure  PROCDEL_BD_EMPRESA (
  pIdEmpresa In   EMPRESA.IdEmpresa%type,
  Rsp        Out  Nocopy Varchar2)
Is
param_input    Varchar2(300);  
codi_error     Integer;
num_reg        Integer;
reg_ja_baixa   exception;
reg_inexistent exception;

Begin   
   param_input := 'pIdEmpresa=' || pIdEmpresa;
   Rsp := 'OK';

-- Comprovem si existeix l'empresa.
   SELECT count(idEmpresa) INTO num_reg FROM EMPRESA WHERE idEmpresa = pIdEmpresa;
   IF num_reg <> 0 THEN
-- La donem de baixa si ja no hi est.
      UPDATE MOLIVERBES.EMPRESA SET data_baixa = comprovacio_data() WHERE IdEmpresa = pIdEmpresa AND data_baixa IS NULL;
      codi_error := SQLCODE;
      IF (SQL%ROWCOUNT = 0) THEN
          raise reg_inexistent;
      END IF;
   ELSE
      raise reg_ja_baixa;
   END IF;
   INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                      VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
   Dbms_output.Put_line(Rsp);
   
   Commit;   

Exception
  WHEN reg_ja_baixa THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Empresa ja de baixa.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Empresa inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/


/*******************************************************************************
Nom    :   PROCINS_BD_COLABORADOR
Paquet :   TFG_BD_ABM
Funci :   Alta d'un nou collaborador
********************************************************************************/
Create Or Replace Procedure PROCINS_BD_COLABORADOR (
  pNom        In   COLABORADOR.NomColab%Type,
  pPerfil     In   COLABORADOR.idPerfil%Type,
  pAdreca     In   COLABORADOR.Adreca%Type,
  pIdPoblacio In   COLABORADOR.idPoblacio%Type,
  pTelefon    In   COLABORADOR.Telefon%Type,
  pMail       In   COLABORADOR.Mail%Type,
  Rsp         Out  Nocopy Varchar2)
Is

param_input Varchar2(300);  
codi_error  Integer;

Begin
  param_input := 'pNom='|| pNom || ' pPerfil=' || pPerfil || ' pAdreca=' || pAdreca || ' pIdPoblacio=' || pIdPoblacio || ' pTelefon=' || pTelefon || ' pMail=' || pMail;
  Rsp := 'OK';
  
  INSERT INTO MOLIVERBES.COLABORADOR(NomColab, IdPerfil, Adreca, IdPoblacio, Telefon, Mail,data_alta) 
                             VALUES (pNom, pPerfil, pAdreca, pIdPoblacio, pTelefon, pMail,comprovacio_data());
  codi_error := SQLCODE;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCUPD_BD_COLABORADOR
Paquet :   TFG_BD_ABM
Funci :   Modifica les dades d'un collaborador
********************************************************************************/
Create Or Replace Procedure  PROCUPD_BD_COLABORADOR (
  pIdColaborador In   COLABORADOR.IdColaborador%Type,
  pNom           In   COLABORADOR.NomColab%Type,
  pAdreca        In   COLABORADOR.Adreca%Type,
  pIdPoblacio    In   COLABORADOR.idPoblacio%Type,
  pTelefon       In   COLABORADOR.Telefon%Type,
  pMail          In   COLABORADOR.Mail%Type,
  Rsp            Out  Nocopy Varchar2)
Is
param_input    Varchar2(300); 
codi_error     Integer;
reg_ja_baixa   exception;
reg_inexistent exception;
num_reg        Integer;

Begin
  param_input := 'pIdColaborador='|| pIdColaborador || 'pNom='|| pNom  || ' pAdreca=' || pAdreca || ' pIdPoblacio=' || pIdPoblacio || ' pTelefon=' || pTelefon || ' pMail=' || pMail;
  Rsp := 'OK';

-- Comprovem si existeix el collaborador
  SELECT count(idColaborador) INTO num_reg FROM COLABORADOR WHERE idColaborador = pIdColaborador;
  IF (num_reg <> 0) THEN
-- Feim l'actualitzaci si no est de baixa
    UPDATE MOLIVERBES.COLABORADOR SET NomColab = pNom, Adreca = pAdreca, IdPoblacio = pIdPoblacio, Telefon = pTelefon, Mail = pMail 
        WHERE IdColaborador = pIdColaborador and data_baixa is NULL;
    codi_error := SQLCODE;
  	IF (SQL%ROWCOUNT = 0) THEN
        raise reg_ja_baixa;
	  END IF;
  ELSE
	   raise reg_inexistent;
  END IF;
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_ja_baixa THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Collaborador de baixa. No es pot modificar';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Collaborador inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCDEL_BD_COLABORADOR
Paquet :   TFG_BD_ABM
Funci :   Elimina un collaborador (Baixa Lgica)
********************************************************************************/
Create Or Replace Procedure  PROCDEL_BD_COLABORADOR (
  pIdColaborador In   COLABORADOR.IdColaborador%type,
  Rsp            Out  Nocopy Varchar2)
Is
param_input    Varchar2(300);  
codi_error     Integer;
num_reg        Integer;
reg_ja_baixa   exception;
reg_inexistent exception;

Begin
   param_input := 'pIdColaborador=' || pIdColaborador;
   Rsp := 'OK';

-- Comprovem si existeix el collaborador
  SELECT count(idColaborador) INTO num_reg FROM COLABORADOR WHERE idColaborador = pIdColaborador;
  IF (num_reg <> 0) THEN
-- El donem de baixa si ja no hi est
      UPDATE MOLIVERBES.COLABORADOR SET data_baixa = comprovacio_data() WHERE IdColaborador = pIdColaborador AND data_baixa IS NULL;
      codi_error := SQLCODE;
      IF (SQL%ROWCOUNT = 0) THEN
        raise reg_ja_baixa;
      END IF;
  ELSE
      raise reg_inexistent;
  END IF;

   INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
   Dbms_output.Put_line(Rsp);
   
   Commit;

Exception
  WHEN reg_ja_baixa THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Collaborador ja de baixa.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Collaborador inexistent';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCINS_BD_ALGORITME
Paquet :   TFG_BD_ABM
Funci :   Alta d'un nou Algoritme
********************************************************************************/
Create Or Replace Procedure PROCINS_BD_ALGORITME (
  pAlgoritme In   ALGORITME.Algoritme%Type,
  Rsp        Out  Nocopy Varchar2)
Is

param_input Varchar2(300);  
codi_error  Integer;


Begin
  param_input := 'pAlgoritme='|| pAlgoritme;
  Rsp := 'OK';

  INSERT INTO MOLIVERBES.ALGORITME(Algoritme) VALUES (pAlgoritme);
  codi_error := SQLCODE;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/


/*******************************************************************************
Nom    :   PROCUPD_BD_ALGORITME
Paquet :   TFG_BD_ABM
Funci :   Modifica les dades d'un algoritme
********************************************************************************/
Create Or Replace Procedure  PROCUPD_BD_ALGORITME (
  pIdAlgoritme In   ALGORITME.IdAlgoritme%Type,
  pAlgoritme   In   ALGORITME.Algoritme%Type,
  Rsp          Out  Nocopy Varchar2)
Is
param_input     Varchar2(300); 
codi_error      Integer;
reg_ja_tramitat exception;
reg_inexistent  exception;
num_reg         Integer;

Begin
  param_input := 'pIdAlgoritme='|| pIdAlgoritme || 'pAlgoritme='|| pAlgoritme;
  Rsp := 'OK';
  num_reg := -1;

-- Comprovem si existeix l'algoritme
  SELECT COUNT(idAlgoritme) INTO num_reg FROM ALGORITME WHERE idAlgoritme = pIdAlgoritme;
  IF num_reg = 0 THEN
    	raise reg_inexistent;
  ELSE
-- Mirem si l'algoritme pertany a algun mes ja tramitat
      SELECT COUNT(A.idAlgoritme) INTO num_reg FROM algoritme A LEFT OUTER JOIN T_Mes M ON A.idAlgoritme = M.idAlgoritme WHERE A.idAlgoritme = pIdAlgoritme AND M.estat = 'S';
      IF num_reg = 0 THEN 
          UPDATE MOLIVERBES.ALGORITME SET Algoritme = pAlgoritme	WHERE IdAlgoritme = pIdAlgoritme;
          codi_error := SQLCODE;
      ELSE
          raise reg_ja_tramitat;
      END IF;
  END IF;
  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;  

Exception
  WHEN reg_ja_tramitat THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Mes ja tramitat';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Algoritme inexistent';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCDEL_BD_ALGORITME
Paquet :   TFG_BD_ABM
Funci :   Elimina un algoritme sempre i quan encara no s'hagi utilitzat
********************************************************************************/
Create Or Replace Procedure  PROCDEL_BD_ALGORITME (
  pIdAlgoritme In   ALGORITME.IdAlgoritme%Type,
  Rsp          Out  Nocopy Varchar2)
Is
param_input      Varchar2(300); 
codi_error       Integer;
reg_ja_tramitat  exception;
reg_inexistent   exception;
num_reg          integer;

Begin
  param_input := 'pIdAlgoritme='|| pIdAlgoritme;
  Rsp := 'OK';
  num_reg := -1;

-- Comprovem si l'algoritme existeix.
  SELECT COUNT(idAlgoritme) INTO num_reg FROM ALGORITME WHERE idAlgoritme = pIdAlgoritme;
  IF num_reg = 0 THEN
    	raise reg_inexistent;
  ELSE 
-- Comprovem si l'algoritme pertany a algun mes ja tramitat.
      SELECT COUNT(A.idAlgoritme) INTO num_reg FROM algoritme A LEFT OUTER JOIN T_Mes M ON A.idAlgoritme = M.idAlgoritme WHERE A.idAlgoritme = pIdAlgoritme AND M.estat = 'S';
      IF num_reg = 0 THEN 
          DELETE FROM MOLIVERBES.ALGORITME WHERE IdAlgoritme = pIdAlgoritme;
          codi_error := SQLCODE;
      ELSE
          raise reg_ja_tramitat;
      END IF;
  END IF;
  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;  

Exception
  WHEN reg_ja_tramitat THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Algoritme pertanyent a un mes ja tramitat';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Algoritme inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/


/*******************************************************************************
Nom    :   PROCINS_BD_EMPRESATIPUSSERVEI
Paquet :   TFG_BD_ABM
Funci :   Relaciona una empresa amb un tipus de Servei
********************************************************************************/
Create Or Replace Procedure  PROCINS_BD_EMPRESATIPUSSERVEI (
  pIdEmpresa    In   EMPRESATIPUSSERVEI.IdEmpresa%Type,
  pIdTipusServei In   EMPRESATIPUSSERVEI.IdTipusServei%Type, 
  Rsp           Out  Nocopy Varchar2)
Is
param_input Varchar2(300);  
codi_error  Integer;

Begin
   param_input := 'pIdEmpresa=' || pIdEmpresa || 'pIdTipusServei=' || pIdTipusServei;
   Rsp := 'OK';

   INSERT INTO MOLIVERBES.EMPRESATIPUSSERVEI (IdEmpresa, IdTipusServei) VALUES (pIdEmpresa, pIdTipusServei); 
   codi_error := SQLCODE;
   
   INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
   Dbms_output.Put_line(Rsp);
   
   Commit;

Exception
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCDEL_BD_EMPRESATIPUSSERVEI
Paquet :   TFG_BD_ABM
Funci :   Elimina una relaci entre Empresa i Tipus de Servei
********************************************************************************/
Create Or Replace Procedure PROCDEL_BD_EMPRESATIPUSSERVEI (
  pIdEmpresa    In   EMPRESATIPUSSERVEI.IdEmpresa%Type,
  pIdTipusServei In   EMPRESATIPUSSERVEI.IdTipusServei%Type, 
  Rsp           Out  Nocopy Varchar2)
Is

param_input    Varchar2(300);  
codi_error     Integer;
reg_inexistent exception;

Begin
  param_input := 'pIdEmpresa=' || pIdEmpresa || 'pIdTipusServei=' || pIdTipusServei;
  Rsp := 'OK';
  
  DELETE FROM MOLIVERBES.EMPRESATIPUSSERVEI WHERE idEmpresa = pIdEmpresa AND idTipusServei = pIdTipusServei;
  codi_error := SQLCODE;
  IF (SQL%ROWCOUNT = 0) THEN
    raise reg_inexistent;
  END IF;
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Relaci empresa/tipus Servei inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
End;
/


/*******************************************************************************
Nom    :   PROCINS_BD_VALORACIOCOLAB
Paquet :   TFG_BD_ABM
Funci :   Modifica les dades d'una valoraci
********************************************************************************/
Create Or Replace Procedure  PROCINS_BD_VALORACIOCOLAB (
  pIdEmpresa     In   VALORACIOCOLABORADORS.IdEmpresa%Type,
  pIdColaborador In   VALORACIOCOLABORADORS.IdColaborador%Type,
  pIdMes         In   VALORACIOCOLABORADORS.IdMes%Type,
  pPunts         In   VALORACIOCOLABORADORS.Punts%Type,
  pComentari     In   VALORACIOCOLABORADORS.Comentari%Type,
  Rsp            Out  Nocopy Varchar2)
Is
param_input     Varchar2(300); 
codi_error      Integer;
Rsp_1           Varchar2(150);
v_perfil        colaborador.idPerfil%Type;
v_mes           t_mes.mes%Type;
v_any           t_any.anny%Type;
num_reg         Integer;
reg_ja_tramitat exception;
mes_posterior   exception; 
val_duplicada   exception;
reg_ja_baixa    exception;

Begin
  param_input := 'pIdEmpresa=' || pIdEmpresa || ' pIdColaborador=' || pIdColaborador || ' pIdMes=' || pIdMes || ' pPunts=' || pPunts || ' pComentari=' ||pComentari;
  Rsp := 'OK';

-- Comprovem que el mes no es posterior a l'actual, ni est ja tramitat.
  SELECT count(idMes) INTO num_reg FROM T_MES WHERE idMes = pIdMes AND estat = 'N';
  IF num_reg = 0 THEN
	   raise reg_ja_tramitat;
  END IF;
  SELECT mes, anny INTO v_mes,v_any FROM T_MES m LEFT OUTER JOIN T_ANY a ON m.idAny = a.idAny WHERE m.idMes = pIdMes;
  IF (v_any > EXTRACT (YEAR FROM SYSDATE)) OR (v_any = EXTRACT(YEAR FROM SYSDATE) AND v_mes > EXTRACT(MONTH FROM SYSDATE)) THEN
	   raise mes_posterior;
  END IF;

-- Si la valoraci pertany a una aplicaci externa, no hi pot haver ms d'una valoraci per mes de la mateixa empresa
  SELECT idPerfil INTO v_perfil FROM COLABORADOR WHERE idColaborador = pIdColaborador;
  IF v_perfil = 'A' THEN
	    SELECT count(idValoracio) INTO num_reg FROM VALORACIOCOLABORADORS WHERE idEmpresa = pIdEmpresa AND idColaborador = pIdColaborador AND idMes = pIdMes;
	    IF num_reg <>0 THEN
		    raise val_duplicada;
	    END IF;
  END IF;

-- No es poden introduir valoracions si el colaborador o l'empresa estan de baixa.
  SELECT count(idEmpresa) INTO num_reg FROM Empresa WHERE idEmpresa = pIdEmpresa AND data_baixa IS NULL; 
  IF num_reg = 0 THEN
	   raise reg_ja_baixa;
  END IF;

  SELECT count(idColaborador) INTO num_reg FROM Colaborador WHERE idColaborador = pIdColaborador AND data_baixa IS NULL;
  IF num_reg = 0 THEN
	   raise reg_ja_baixa;
  END IF;

-- Si no hem fet saltar cap excepci, podem introduir la valoraci a la taula
  INSERT INTO MOLIVERBES.VALORACIOCOLABORADORS(IdEmpresa, IdColaborador, IdMes, Punts, Comentari) VALUES (pIdEmpresa, pIdColaborador, pIdMes, pPunts, pComentari);
  codi_error := SQLCODE;
	
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_ja_tramitat THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: No es poden introduir ms valoracions per a aquest mes. Mes tramitat.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN mes_posterior THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: No es poden introduir valoracions de mesos posteriors.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN val_duplicada THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: No hi pot haver ms de una valoraci per mes per a les aplicacions externes.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN reg_ja_baixa THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Empresa o Collaborador donat de baixa';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
End;
/


/*******************************************************************************
Nom    :   PROCUPD_BD_VALORACIOCOLAB
Paquet :   TFG_BD_ABM
Funci :   Modifica una valoraci feta per un collaborador, sempre i quan no
	   hagi estat tramitada
********************************************************************************/
Create Or Replace Procedure  PROCUPD_BD_VALORACIOCOLAB (
  pIdValoracio In   VALORACIOCOLABORADORS.IdValoracio%Type,
  pPunts       In   VALORACIOCOLABORADORS.Punts%Type,
  pComentari   In   VALORACIOCOLABORADORS.Comentari%Type,
  Rsp          Out  Nocopy Varchar2)
Is
param_input     Varchar2(300);  
codi_error      Integer;
reg_ja_tramitat exception;
num_reg         integer;
reg_inexistent  exception;

Begin
  param_input := 'pIdValoracio=' || pIdValoracio || ' pPunts=' || pPunts;
  Rsp := 'OK';
  num_reg := -1;

-- Comprovem si existeix la valoraci
   SELECT count(idValoracio) INTO num_reg FROM ValoracioColaboradors WHERE idValoracio = pIdValoracio;
   IF num_reg = 0 THEN
      raise reg_inexistent;
   ELSE
-- Comprovem si la valoraci ja ha estat tramitada.
      SELECT count(V.idValoracio) INTO num_reg FROM T_Mes M LEFT OUTER JOIN ValoracioColaboradors V ON M.idMes = V.idMes
             WHERE V.idValoracio = pIdValoracio AND M.estat = 'N';
      IF num_reg = 0 THEN
         raise reg_ja_tramitat;
      ELSE
         UPDATE MOLIVERBES.VALORACIOCOLABORADORS SET Punts = pPunts, Comentari = pComentari WHERE IdValoracio = pIdValoracio; 
         codi_error := SQLCODE;
      END IF;
   END IF;
   
   INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
   Dbms_output.Put_line(Rsp);
   
   Commit;      

Exception
  WHEN reg_ja_tramitat THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Mes ja tramitat. No es pot modificar.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Valoraci inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
End;
/

/*******************************************************************************
Nom    :   PROCDEL_BD_VALORACIOCOLAB
Paquet :   TFG_BD_ABM
Funci :   Elimina una puntuaci feta per un collaborador sempre i quan no hagi
	   estat tramitada
********************************************************************************/
Create Or Replace Procedure PROCDEL_BD_VALORACIOCOLAB (
  pIdValoracio In   VALORACIOCOLABORADORS.IdValoracio%Type,
  Rsp          Out  Nocopy Varchar2)
Is

param_input     Varchar2(300);  
codi_error      Integer;
reg_ja_tramitat exception;
num_reg         integer;
reg_inexistent  exception;

Begin
  param_input := 'pIdValoracio=' || pIdValoracio;
  Rsp := 'OK';
  num_reg := -1;

-- Comprovem si existeix la valoraci
   SELECT count(idValoracio) INTO num_reg FROM ValoracioColaboradors WHERE idValoracio = pIdValoracio;
   IF num_reg = 0 THEN 
      raise reg_inexistent;
   ELSE
      SELECT count(idValoracio) INTO num_reg FROM T_Mes M LEFT OUTER JOIN ValoracioColaboradors V ON M.idMes = V.idMes
             WHERE V.idValoracio = pIdValoracio AND M.Estat = 'N';
      IF num_reg = 0 THEN
         raise reg_ja_tramitat;
      ELSE
         DELETE FROM MOLIVERBES.VALORACIOCOLABORADORS WHERE IdValoracio = pIdValoracio;
         codi_error := SQLCODE;
         
      END IF;
   END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_ja_tramitat THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Mes ja tramitat. No es pot eliminar';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Valoraci inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
End;
/

/*******************************************************************************
Nom   :   PROCINS_BD_MES
Paquet:   TFG_BD_ABM
Funcio:   Alta d'un nou mes
********************************************************************************/
Create Or Replace Procedure PROCINS_BD_MES (
  pMes         In   T_MES.Mes%Type,
  pIdAny       In   T_MES.idAny%Type,
  pIdAlgoritme In   T_MES.idAlgoritme%Type,
  Rsp          Out  Nocopy Varchar2)
Is

param_input Varchar2(300);  
codi_error  Integer;

Begin
  param_input := 'pMes=' || pMes || ' pIdAny=' || pIdAny || ' pIdAlgoritme=' || pIdAlgoritme;
  Rsp := 'OK';
  
  INSERT INTO MOLIVERBES.T_MES(mes, idAny, idAlgoritme) VALUES (pMes, pIdAny, pIdAlgoritme);
  codi_error := SQLCODE;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
End;
/

/*******************************************************************************
Nom   :   PROCUPD_BD_MES
Paquet:   TFG_BD_ABM
Funcio:   Permet modificar l'algorimte d'aquell mes
********************************************************************************/
Create Or Replace Procedure PROCUPD_BD_MES (
  pIdMes       In   T_MES.IdMes%Type,
  pIdAlgoritme In   T_MES.idAlgoritme%Type,
  Rsp          Out  Nocopy Varchar2)
Is

param_input     Varchar2(300);  
codi_error      Integer;
num_reg         Integer;
reg_ja_tramitat exception;
reg_inexistent  exception;

Begin
  param_input := 'pIdMes=' || pIdMes || ' pIdAlgoritme=' || pIdAlgoritme;
  Rsp := 'OK';
  
  SELECT count(idMes) INTO num_reg FROM T_Mes WHERE idMes  = pIdMes;
  IF num_reg = 0 THEN
     raise reg_inexistent;
  ELSE
     UPDATE MOLIVERBES.T_MES SET idAlgoritme = pIdAlgoritme WHERE idMes = pIdMes AND estat = 'N';
     codi_error := SQLCODE;
     IF (SQL%ROWCOUNT = 0) THEN
          raise reg_ja_tramitat;
     END IF;
  END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_ja_tramitat THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Mes amb valoracions ja tramitat. No es pot modificar.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Mes inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
End;
/

/*******************************************************************************
Nom   :   PROCDEL_BD_MES
Paquet:   TFG_BD_ABM
Funcio:   Elimina un mes
********************************************************************************/
Create Or Replace Procedure PROCDEL_BD_MES (
  pIdMes In   T_MES.IdMes%Type,
  Rsp    Out  Nocopy Varchar2)
Is

param_input     Varchar2(300);  
codi_error      Integer;
num_reg         Integer;
reg_ja_tramitat exception;
reg_inexistent  exception;

Begin
  param_input := 'pIdMes=' || pIdMes;
  Rsp := 'OK';
  
  SELECT count(idMes) INTO num_reg FROM T_MES WHERE idMes  = pIdMes;
  IF num_reg = 0 THEN
    	raise reg_inexistent;
  ELSE
-- Eliminam el mes. Saltar restricci d'integritat si el mes est relacionat amb qualsevol altra taula.
      DELETE FROM MOLIVERBES.T_MES WHERE idMes = pIdMes; 
      codi_error := SQLCODE;
  END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Mes inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
End;
/


/*******************************************************************************
Nom   :   PROCINS_BD_ANY
Paquet:   TFG_BD_ABM
Funcio:   Alta d'un nou any
********************************************************************************/
Create Or Replace Procedure PROCINS_BD_ANY (
  pAny In   T_ANY.Anny%Type,
  Rsp  Out  Nocopy Varchar2)
Is

param_input Varchar2(300);  
codi_error  Integer;

Begin
  param_input := 'pAny=' || pAny;
  Rsp := 'OK';
  
  INSERT INTO MOLIVERBES.T_ANY(Anny) VALUES (pAny);
  codi_error := SQLCODE;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN DUP_VAL_ON_INDEX THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
End;
/

/*******************************************************************************
Nom   :   PROCUPD_BD_ANY
Paquet:   TFG_BD_ABM
Funcio:   Modificaci d'un any
********************************************************************************/
Create Or Replace Procedure PROCUPD_BD_ANY (
  pIdAny In   T_ANY.idAny%Type,
  pAny   In   T_ANY.Anny%Type,
  Rsp    Out  Nocopy Varchar2)
Is

param_input     Varchar2(300);  
codi_error      Integer;
num_reg         Integer;
reg_ja_tramitat exception;
reg_inexistent  exception;

Begin
  param_input := 'pIdAny=' || pIdAny || 'pAny=' || pAny;
  Rsp := 'OK';

-- Comprovem si l'any existeix
  SELECT COUNT(idAny) INTO num_reg FROM T_ANY WHERE IdAny = pIdAny;
  IF num_reg = 0 THEN
     raise reg_inexistent;
  ELSE
-- Comprovem si l'any t alguna valoraci introduda.
     SELECT count(V.idValoracio) INTO num_reg FROM ValoracioColaboradors V
            LEFT OUTER JOIN T_MES M ON M.IDMES = V.IDMES
            LEFT OUTER JOIN T_ANY A ON A.IDANY = M.IDANY
            WHERE A.idAny = pIdAny;
     IF num_reg <> 0 THEN
        raise reg_ja_tramitat;
     ELSE
    	  UPDATE MOLIVERBES.T_ANY SET Anny = pAny WHERE idAny = pIdAny;
        codi_error := SQLCODE;
     END IF;
  END IF;
  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_ja_tramitat THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Any amb valoracions. No es pot modificar';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Any inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN ROWTYPE_MISMATCH THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
End;
/

/*******************************************************************************
Nom   :   PROCDEL_BD_ANY
Paquet:   TFG_BD_ABM
Funcio:   Elimina fsicament un any
********************************************************************************/
Create Or Replace Procedure PROCDEL_BD_ANY (
  pIdAny In   T_ANY.idAny%Type,
  Rsp    Out  Nocopy Varchar2)
Is

param_input     Varchar2(300);  
codi_error      Integer;
num_reg         Integer;
reg_ja_tramitat exception;
reg_inexistent  exception;

Begin
  param_input := 'pIdAny=' || pIdAny;
  Rsp := 'OK';
  
-- Comprovem si l'any existeix
  SELECT COUNT(idAny) INTO num_reg FROM T_Any WHERE IdAny = pIdAny;
  IF num_reg = 0 THEN  
     raise reg_inexistent;
  ELSE
-- Comprovem si l'any t alguna valoraci introduda.
     SELECT count(V.idValoracio) INTO num_reg FROM ValoracioColaboradors V
            LEFT OUTER JOIN T_MES M ON M.IDMES = V.IDMES
            LEFT OUTER JOIN T_ANY A ON A.IDANY = M.IDANY
            WHERE A.idAny = pIdAny;
     IF num_Reg <> 0 THEN
        raise reg_ja_tramitat;
     ELSE
    	  DELETE FROM MOLIVERBES.T_ANY WHERE idAny = pIdAny;
        codi_error := SQLCODE;
     END IF;
  END IF;
  
  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN reg_ja_tramitat THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Any amb valoracions. No es pot eliminar';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN reg_inexistent THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: Any inexistent.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= SUBSTR('ERROR: ' || codi_error ||' ' || SQLERRM ,1,300);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error); 
    Commit;
End;
/

commit;