/* IMPORTANT: Es necessari estar connectat amb l'usuari MOLIVERBES i password SECRET     */
/* ----------  abans d'executar aquest script                                             */
set serveroutput on;

/*****************************************************************************/
/*                 CREACIO DE FUNCIONS I PROC. EMMAGATZEMATS                 */
/*****************************************************************************/

/*******************************************************************************
Nom:       PROC_PUNTS_EMPRESA
Paquet:    TFG_PK_DW
Funcio:    Llista la puntuaci mensual de cada empresa de restauraci
********************************************************************************/
CREATE OR REPLACE PROCEDURE PROC_PUNTS_EMPRESA(
  Rsp Out Nocopy Varchar2)
IS

param_input    Varchar2(300); 
codi_error     Integer;
no_reg         boolean := true;
print_cap      boolean := true;
no_reg_trobats exception;

CURSOR curs  IS
  SELECT nom, punts, mes, anny 
  FROM dades_empresa_mes P
  LEFT OUTER JOIN Empresa E ON E.idEmpresa = P.idEmpresa
  LEFT OUTER JOIN T_Mes M ON P.idMes = M.idMes
  LEFT OUTER JOIN T_Any A ON M.idAny = A.idANy
  ORDER BY e.idEmpresa;

BEGIN 
param_input := '' ;
Rsp := 'OK';

FOR i in curs LOOP
  no_reg := false;
  IF print_cap THEN
	   dbms_output.put_line('Puntuaci mensual de cada empresa de restauraci');
	   dbms_output.put_line('--------------------------------------------------');
	   dbms_output.put_line('');
	   dbms_output.put_line('MES      EMPRESA                                             PUNTS');
	   dbms_output.put_line('------------------------------------------------------------------');
	   print_cap := false;
  END IF;
  dbms_output.put_line(LPAD(i.mes,2,' ')||'/'||i.anny||'  '||RPAD(i.nom,50,' .')||' '||LPAD(to_char(i.punts,'fm9990D0'),5,' '));
END LOOP;
IF no_reg THEN
	raise no_reg_trobats;  
END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN no_reg_trobats THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: No s''han trobat registres que compleixin la condici.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN CURSOR_ALREADY_OPEN THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN INVALID_CURSOR THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom:       PROC_PUNTS_MES_POBLACIO
Paquet:    TFG_PK_DW
Funcio:    Donada una ciutat i un mes, llistat de totes les empreses de
	   restauraci d'aquella ciutat que la seva valoraci estigui entre les
	   10 millors d'aquell mes, ordenades de forma descendent
********************************************************************************/
CREATE OR REPLACE PROCEDURE PROC_PUNTS_MES_POBLACIO(
  pIdMes      In   T_MES.idMes%Type,
  pIdPoblacio In   POBLACIO.idPoblacio%Type,
  Rsp         Out  Nocopy Varchar2)
IS

param_input    Varchar2(300); 
codi_error     Integer;
no_reg         boolean := true;
print_cap      boolean := true;
no_reg_trobats exception;

CURSOR curs  IS
  SELECT * FROM 
       (SELECT nom, punts 
        FROM dades_empresa_mes P
        LEFT OUTER JOIN empresa E ON P.idEmpresa = E.idEmpresa
        WHERE idMes = pidMes AND idPoblacio = pidPoblacio
        ORDER BY punts DESC)
  WHERE ROWNUM<=10;
  
BEGIN 
param_input := 'pIdMes= ' || pIdMes || ' pIdPoblacio= ' || pIdPoblacio ;
Rsp := 'OK';


FOR i in curs LOOP
  no_reg := false;
  IF print_cap THEN
     dbms_output.put_line('10 empreses ms valorades de la poblaci durant el mes ');
	   dbms_output.put_line('-------------------------------------------------------');
	   dbms_output.put_line('EMPRESA                                             PUNTS');
	   dbms_output.put_line('-----------------------------------------------------------');
	   print_cap := false;
  END IF;
  dbms_output.put_line(RPAD(i.nom,50,' .')||' '||LPAD(to_char(i.punts,'fm9990D0'),5,' '));
END LOOP;
IF no_reg THEN
	raise no_reg_trobats;
END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN no_reg_trobats THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: No s''han trobat registres que compleixin la condici.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN CURSOR_ALREADY_OPEN THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN INVALID_CURSOR THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;

/

/*******************************************************************************
Nom:       PROC_DIFERENCIA_INS_AEX
Paquet:    TFG_PK_DW
Funcio:    Llistat de les 10 empreses de restauraci en les que, en el mes
	   anterior a l'actual, sigui ms gran la diferncia entre la mitjana 
	   de valoraci dels inspetors del gremi i la mitjana de valoraci
	   rebuda per les apliacions de valoraci externes.
********************************************************************************/
CREATE OR REPLACE PROCEDURE PROC_DIFERENCIA_INS_AEX(
  Rsp Out Nocopy Varchar2)
IS
  
param_input     Varchar2(300); 
codi_error      Integer;
v_idMesAnterior Integer;
v_mesAnterior   T_Mes.Mes%Type;
v_anyAnterior   T_ANY.ANNY%Type;
no_reg          boolean := true;
print_cap       boolean := true;
no_reg_trobats  exception;


CURSOR curs IS
   SELECT * FROM
         (SELECT nom, difInsAex 
          FROM dades_empresa_mes P
          LEFT OUTER JOIN empresa E ON E.idEmpresa = P.idEmpresa
          WHERE idMes = v_idMesAnterior
          ORDER BY P.difInsAex DESC)
   WHERE ROWNUM<= 10;


BEGIN 
param_input := '' ;
Rsp := 'OK';

   SELECT EXTRACT(Month FROM SYSDATE)-1 INTO v_mesAnterior FROM DUAL;
   IF v_mesAnterior = 0 THEN
      v_mesAnterior := 12;
      v_anyAnterior := EXTRACT(Year FROM SYSDATE)-1;
   ELSE
      v_anyAnterior := EXTRACT(Year FROM SYSDATE);
   END IF;
   
   SELECT idMes INTO v_idMesAnterior FROM T_Mes M
   LEFT OUTER JOIN T_Any A ON M.idAny = A.idAny 
   WHERE mes = v_mesAnterior and Anny =v_AnyAnterior;
   
   FOR i in curs LOOP
      no_reg := false;
      IF print_cap THEN
	       dbms_output.put_line('Diferncia entre la mitja de puntuacions dels inspectors i les aplicacions externes al mes anterior');
	       dbms_output.put_line('-----------------------------------------------------------------------------------------------------');
	       dbms_output.put_line('EMPRESA                                            DIFERENCIA');
	       dbms_output.put_line('--------------------------------------------------------------');
	       print_cap := false;
      END IF;
      dbms_output.put_line(RPAD(i.nom,50,' .')||' '||LPAD(to_char(i.difInsAEx,'fm9990D0'),5,' '));
   END LOOP;

   IF no_reg THEN
	    raise no_reg_trobats;
   END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN no_reg_trobats THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: No s''han trobat registres que compleixin la condici.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN CURSOR_ALREADY_OPEN THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN INVALID_CURSOR THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom:       PROC_TOPEMPRESA
Paquet:    TFG_PK_DW
Funcio:    Empresa de restauraci que ha estat ms mesos com la ms valorada
********************************************************************************/
CREATE OR REPLACE PROCEDURE PROC_TOPEMPRESA(
  Rsp Out Nocopy Varchar2)
IS

param_input Varchar2(300); 
codi_error  Integer;
v_nom       empresa.nom%Type;
v_num       empresa.nombrecopstop%Type;

BEGIN 
param_input := '' ;
Rsp := 'OK';
   SELECT * INTO v_nom,v_num FROM
         (SELECT nom, nombreCopsTop 
          FROM Empresa
          ORDER BY nombreCopsTop DESC)
   WHERE ROWNUM<=1;


   dbms_output.put_line('Empresa de restauraci que ha estat ms mesos com la ms valorada');
   dbms_output.put_line('-----------------------------------------------------------------');
   dbms_output.put_line('');
   dbms_output.put_line('EMPRESA                                            NOMBRE DE MESOS');
   dbms_output.put_line('-------------------------------------------------------------------');
   dbms_output.put_line(RPAD(v_nom,50,' .')||' '||LPAD(v_num,3,' '));

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom:       PROC_MITJA_CLIENTS
Paquet:    TFG_PK_DW
Funcio:    Valoraci mitjana que fan els clients registrats
********************************************************************************/
CREATE OR REPLACE PROCEDURE PROC_MITJA_CLIENTS(
  Rsp Out Nocopy Varchar2)
IS

param_input Varchar2(300); 
codi_error  Integer;
v_mitja     perfil.mitja%Type;

BEGIN 
param_input := '' ;
Rsp := 'OK';

   SELECT mitja INTO v_mitja
          FROM perfil
          WHERE idPerfil = 'C';

   dbms_output.put_line('Valoraci mitjana que fan els clients registrats'); 
   dbms_output.put_line('------------------------------------------------');
   dbms_output.put_line('');
   dbms_output.put_line('VALORACIO MITJANA');
   dbms_output.put_line('-----------------');
   dbms_output.put_line(to_char(v_mitja,'fm9990D0'));

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom:       PROC_MITJA_ANY_POBLACIO
Paquet:    TFG_PK_DW
Funcio:    Valor mig de les valoracions de totes les empreses de restauraci
	   d'una poblaci i any
********************************************************************************/
CREATE OR REPLACE PROCEDURE PROC_MITJA_ANY_POBLACIO(
  pIdAny      In   T_ANY.idAny%Type,
  pIdPoblacio In   POBLACIO.idPoblacio%Type, 
  Rsp         Out  Nocopy Varchar2)
IS

param_input Varchar2(300); 
codi_error  Integer;
v_mitja     dades_poblacio_any.mitja%Type;

BEGIN 
param_input := 'pIdAny= ' || pIdAny || ' pIdPoblacio= ' || pIdPoblacio ;
Rsp := 'OK';

   SELECT mitja INTO v_mitja
          FROM dades_poblacio_any
          WHERE idAny = pidAny AND idPoblacio = pIdPoblacio;

   dbms_output.put_line('Valoraci mitja de totes les empreses de la poblacio durant l''any');
   dbms_output.put_line('-----------------------------------------------------------------');
   dbms_output.put_line('');
   dbms_output.put_line('VALORACIO MITJA');
   dbms_output.put_line('---------------');
   dbms_output.put_line(to_char(v_mitja,'fm9990D0'));

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom:       PROC_NUM_VALOR_CLIENTS
Paquet:    TFG_PK_DW
Funcio:    TOP-10 d'empreses de restauraci pel que fa al nmero de valoracions
	   entrades per clients
********************************************************************************/
CREATE OR REPLACE PROCEDURE PROC_NUM_VALOR_CLIENTS(
  Rsp Out Nocopy Varchar2)
IS

param_input    Varchar2(300); 
codi_error     Integer;
no_reg         boolean := true;
print_cap      boolean := true;
no_reg_trobats exception;

CURSOR curs IS
     SELECT * FROM
            (SELECT nom, numValClients FROM empresa
             ORDER BY numValClients DESC)
     WHERE ROWNUM<=10;

BEGIN 
param_input := '' ;
Rsp := 'OK';

   FOR i in curs LOOP
       no_reg := false;
       IF print_cap THEN
	        dbms_output.put_line('Top10 per nmero de valoracions de clients');
	        dbms_output.put_line('-----------------------------------');
	        dbms_output.put_line('EMPRESA                                            NOMBRE VALORACIONS');
	        dbms_output.put_line('---------------------------------------------------------------------');
	        print_cap := false;
      END IF;
      dbms_output.put_line(RPAD(i.nom,50,' .')||' '||LPAD(i.numValClients,3,' '));
   END LOOP;
   IF no_reg THEN
	    raise no_reg_trobats;
   END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN no_reg_trobats THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: No s''han trobat registres que compleixin la condici.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN CURSOR_ALREADY_OPEN THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN INVALID_CURSOR THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom:       PROC_MITJA_POBLACIO_MES
Paquet:    TFG_PK_DW
Funcio:    Valoraci mitjana de totes les empreses de restauraci per cada 
	   ciutat i mes.
********************************************************************************/
CREATE OR REPLACE PROCEDURE PROC_MITJA_POBLACIO_MES(
  Rsp Out Nocopy Varchar2)
IS

param_input    Varchar2(300); 
codi_error     Integer;
no_reg         boolean := true;
print_cap      boolean := true;
no_reg_trobats exception;

CURSOR curs IS
   SELECT mes, anny, poblacio, mitja 
   FROM dades_poblacio_mes V
   LEFT OUTER JOIN T_Mes M ON V.idMes = M.idMes
   LEFT OUTER JOIN T_Any A ON A.idAny = M.idAny
   LEFT OUTER JOIN Poblacio P ON V.idPoblacio = P.idPoblacio
   ORDER BY A.Anny, M.Mes;

BEGIN 
param_input := '' ;
Rsp := 'OK';

   FOR i IN curs LOOP
      no_reg := false;
      IF print_cap THEN
	       dbms_output.put_line('Valoraci mitjana per poblacio i mes');
         dbms_output.put_line('---------------------------------------');
	       dbms_output.put_line('MES      POBLACIO                         VALORACIO MITJA');
	       dbms_output.put_line('---------------------------------------------------------');
	       print_cap := false;
      END IF;
      dbms_output.put_line(LPAD(i.mes,2,' ')||'/'||i.anny||'  '||RPAD(i.poblacio,30,' .')||' '||LPAD(to_char(i.mitja,'fm9990D0'),5,' '));
   END LOOP;
   IF no_reg THEN
	    raise no_reg_trobats;
   END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN no_reg_trobats THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: No s''han trobat registres que compleixin la condici.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN CURSOR_ALREADY_OPEN THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN INVALID_CURSOR THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom:       PROC_MITJA_INSPECTOR_EMPRESA
Paquet:    TFG_PK_DW
Funcio:    TOP5 Inspectors pel que va a la seva puntuaci mitjana per empresa
********************************************************************************/
CREATE OR REPLACE PROCEDURE PROC_MITJA_INSPECTOR_EMPRESA(
  Rsp Out Nocopy Varchar2)
IS

param_input    Varchar2(300); 
codi_error     Integer;
no_reg         boolean := true;
print_cap      boolean := true;
no_reg_trobats exception;

CURSOR curs IS
  SELECT * FROM
      (SELECT E.nom, C.nomColab, V.mitja 
       FROM dades_colab_empresa V
       LEFT OUTER JOIN empresa E ON V.idEmpresa = E.idEmpresa
       LEFT OUTER JOIN colaborador C ON V.idColaborador = C.idColaborador
       WHERE C.idPerfil = 'I'
       ORDER BY E.idEmpresa, V.mitja DESC)
  WHERE ROWNUM<=5;

BEGIN 
param_input := '' ;
Rsp := 'OK';

   FOR i IN curs LOOP
      no_reg := false;
      IF print_cap THEN
	       dbms_output.put_line('TOP5 valoracions de inspectors per empresa');
         dbms_output.put_line('--------------------------------------------');
	       dbms_output.put_line('EMPRESA                                            INSPECTOR                                         MITJA');
         dbms_output.put_line('----------------------------------------------------------------------------------------------------------');
	       print_cap := false;
      END IF;
      dbms_output.put_line(RPAD(i.nom,50,' .')||' '||RPAD(i.nomColab,50,' .')||' '||LPAD(to_char(i.mitja,'fm9990D0'),5,' '));
   END LOOP;
   IF no_reg THEN
	    raise no_reg_trobats;
   END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN no_reg_trobats THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: No s''han trobat registres que compleixin la condici.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN NO_DATA_FOUND THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

/*******************************************************************************
Nom:       PROC_MITJA_CATEGORIA_TIPUS
Paquet:    TFG_PK_DW
Funcio:    TOP10 d'empreses de restauraci pel que fa al valor de valoracions
	   per a cada categoria i tipologia d'empresa.
********************************************************************************/
CREATE OR REPLACE PROCEDURE PROC_MITJA_CATEGORIA_TIPUS(
  Rsp Out Nocopy Varchar2)
IS

param_input    Varchar2(300); 
codi_error     Integer;
no_reg         boolean := true;
print_cap      boolean := true;
no_reg_trobats exception;

CURSOR curs IS
   select * from
          (select 
           row_number() over (partition by c.idCategoria,t.idTipusservei order by e.mitja desc) x,
           c.categoria, 
           t.tipus,
           e.nom,
           e.mitja
           from empresa e
           left outer join categoria c on c.idcategoria = e.idCategoria
           left outer join empresatipusservei x on x.idempresa = e.idempresa
           left outer join tipusservei t on t.idtipusservei = x.idtipusservei
           order by c.idcategoria, t.tipus, mitja desc) alias
   where x <= 10;

BEGIN 
param_input := '' ;
Rsp := 'OK';

   FOR i IN curs LOOP
      no_reg := false;
      IF print_cap THEN
	       dbms_output.put_line('TOP 10 per tipologia i categoria');
	       dbms_output.put_line('-------------------------------');
         dbms_output.put_line('CATEGORIA                  TIPOLOGIA             EMPRESA                                             MITJA');
	       dbms_output.put_line('----------------------------------------------------------------------------------------------------------');
	       print_cap := false;
      END IF;
      dbms_output.put_line(RPAD(i.categoria,25,' .')||' '||RPAD(i.tipus,20,' .')||' '||RPAD(i.nom,50,' .')||' '||LPAD(to_char(i.mitja,'fm9990D0'),5,' '));
   END LOOP;
   IF no_reg THEN
	    raise no_reg_trobats;
   END IF;

  INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) 
                     VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
  Dbms_output.Put_line(Rsp);
  
  Commit;

Exception
  WHEN no_reg_trobats THEN
    codi_error := SQLCODE;
    Rsp:= 'ERROR: No s''han trobat registres que compleixin la condici.';
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN CURSOR_ALREADY_OPEN THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN INVALID_CURSOR THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
  WHEN Others THEN
    codi_error := SQLCODE;
    Rsp:= Substr('ERROR: ' || codi_error ||' ' || SQLERRM ,1,50);
    Dbms_output.Put_line(Rsp);
    Rollback;
    INSERT INTO MOLIVERBES.LOG(PROCEDIMENT, PARAMETRES, RSP, CODIERROR) VALUES ($$PLSQL_UNIT, SUBSTR(param_input,1,300), Rsp, codi_error);
    Commit;
End;
/

COMMIT;

